#### Install packages (if necessary) ####
install.packages("tidyverse")
install.packages("scales")
install.packages("minpack.lm")
install.packages("parallel")
# devtools requires Rtools. Please see https://www.r-project.org/nosvn/pandoc/devtools.html
install.packages("devtools") 
devtools::install_github("mjlacey/impedanceR")
install.packages("viridis")
install.packages("ggthemes")

#### Packages checks ####
# Start here if packages are installed
library(tidyverse)
library(scales)
library(minpack.lm)
library(parallel)
library(impedanceR)
library(viridis)
library(ggthemes)

#### Get the plotting theme and functions ####
# Please make sure that the working directory is the same as the directory of this script.
source("../R_functions_themes/theme.R")
source("../R_functions_themes/functions.R")

#### Import data ####
## Import mpt files and sort them into DC, statistics and AC parts ##
# Stitch HL_CCI811_4 and HL_CCI811_4-2 due to an accidental interruption of the test
cell2.1 <- read_tsv("HL_CCI811_4_02_MB_C02.mpt", skip =  as.numeric(gsub("Nb header lines : ", "", readLines("HL_CCI811_4_02_MB_C02.mpt", n = 2)[2]))-1)
cell2.2 <- read_tsv("HL_CCI811_4-2_02_MB_C02.mpt", skip =  as.numeric(gsub("Nb header lines : ", "", readLines("HL_CCI811_4-2_02_MB_C02.mpt", n = 2)[2]))-1)%>% 
        mutate(`time/s` = `time/s` + last(cell2.1$`time/s`), `z cycle` = `z cycle` + last(cell2.1$`z cycle`), `(Q-Qo)/mA.h` = `(Q-Qo)/mA.h` + last(cell2.1$`(Q-Qo)/mA.h`))
cell2 <- bind_rows(cell2.1, tail(cell2.2, -9))
cell2.1 <- NULL
cell2.2 <- NULL

# Import data from the modified GITT program for the comparison between GITT, ICI and EIS
data.1 <- import_biologic(file = "HL_CCI811_43V_ICI-EIS_3E182_02_MB_C16.mpt", mass = 2.8, three.electrode = TRUE)
data.2 <- import_biologic(data = cell2, mass = 2.824, three.electrode = TRUE) 
cell2 <- NULL

## Import data from the subsequent ICI cycles ##
# Skip this part and run it alone if R terminates due to memory constraints.
data.1_ici <- import_biologic(file = "HL_CCI811_43V_ICI-EIS_3E182_03_MB_C16.mpt", mass = 2.8, three.electrode = TRUE)
data.2_ici <- import_biologic(file = "HL_CCI811_4-2_03_MB_C02.mpt", mass = 2.824, three.electrode = TRUE)

#### Data analysis ####
## Constants in Equation 2: 4/pi*(V/A)^2 
V <- 0.63056 #cm3/g
A <- 1.5e4 #cm2/g
constant <- 4/pi*(V/A)^2  #cm2

## GITT, ICI and EIS analyses ##
# GITT analysis of Cell 1 with data selection interval 5-40 s
gitt_1 <- filter(data.1$raw, rest != 124) %>% 
  GITT(tmin =5, tmax = 40, three.electrode = TRUE)

# GITT analysis of Cell 1 with data selection interval 50-150 s
gitt_1_150 <- filter(data.1$raw, rest != 124) %>% 
  GITT(tmin =50, tmax = 150, three.electrode = TRUE)

# ICI analysis of Cell 1 in the modified GITT program
gitt_r_1 <- filter(data.1$raw, rest != 191, rest != 65) %>% 
  ICI_np(tmin = 1, tmax = 5, three.electrode = TRUE) %>% 
  merge(., select(gitt_1, rest, E4), by = "rest") # Get the OCP from GITT results for plotting

# GITT analysis of Cell 2 with data selection interval 5-40 s
gitt_2 <- filter(data.2$raw, rest != 65, rest != 108, rest != 125) %>% 
  GITT(tmin = 5, tmax = 40, three.electrode = TRUE)

# GITT analysis of Cell 2 with data selection interval 50-150 s
gitt_2_150 <- filter(data.2$raw, rest != 65, rest != 108, rest != 125) %>% 
  GITT(tmin =50, tmax = 150, three.electrode = TRUE)

# ICI analysis of Cell 2 in the modified GITT program
gitt_r_2 <- filter(data.2$raw, rest != 65, rest != 108, rest != 192) %>% 
  ICI_np(tmin = 1, tmax = 5, three.electrode = TRUE) %>% 
  merge(., select(gitt_2, rest, E4), by = "rest") # Get the OCP from GITT results for plotting

# EIS fitting
# Define equivalent circuit model in Supplementary Fig. 13
ecm <- function(p,x){
  z <- Z_R(p[1])
  z <- z + para(Z_R(p[2]), Z_Q(p[3],p[4],2*pi*x)) 
  z <- z +  para(Z_R(p[5]), Z_Q(p[6],p[7],2*pi*x)) 
  z <- z + Z_W(p[8], 2*pi*x)
  return(z)
}

# Sequential fitting of Cell 1
# Define an empty list
fit1<-vector(mode = "list")
# Input an initial guess of the fitting parameters
p0 <- c(3, 41, 7.9e-5, 0.6, 16, 6.3e-4, 0.98, 7)
# Lower limit of parameters
pl <- c(0,0,0,0,0,0,0,0)
# Upper limit of parameters
pu <- c(100, 1000, 1e-2, 1, 1000, 1e-1, 1, 30)

p<-p0
for (i in 15:43){
  fit1[[i]] <- EIS_fitting(filter(select(data.1$eis[[i]], f, Re, Im), f >0.04), ecm, p, pl, pu)
  p <- fit1[[i]]$par
  print(paste("Complete spectra nr.", i))
}

for (i in 44:90){
  fit1[[i]] <- EIS_fitting(select(data.1$eis[[i]], f, Re, Im), ecm, p, pl, pu)
  p <- fit1[[i]]$par
  print(paste("Complete spectra nr.", i))
}

# adjust parameters since the spectra without a Warburg tail at lower SoC are skipped
p[2]<-p[2]+15
p[5]<-p[5]+7
p[8]<-5
for (i in 124:210){
  fit1[[i]] <- EIS_fitting(filter(select(data.1$eis[[i]], f, Re, Im)), ecm, p, pl, pu)
  p <- fit1[[i]]$par
  print(paste("Complete spectra nr.", i))
}

# Sequential fitting of Cell 2
# Define an empty list
fit2<-vector(mode = "list")
# Input an initial guess of the fitting parameters
p0 <- c(3, 41, 7.9e-5, 0.6, 16, 6.3e-4, 0.98, 7)
# Lower limit of parameters
pl <- c(0,0,0,0,0,0,0,0)
# Upper limit of parameters
pu <- c(100, 1000, 1e-2, 1, 1000, 1e-1, 1, 20)

p<-p0
for (i in 16:43){
  fit2[[i]] <- EIS_fitting(filter(select(data.2$eis[[i]], f, Re, Im), f>0.04), ecm, p, pl, pu)
  p <- fit2[[i]]$par
  print(paste("Complete spectra nr.", i))
}

for (i in 44:94){
  fit2[[i]] <- EIS_fitting(filter(select(data.2$eis[[i]], f, Re, Im)), ecm, p, pl, pu)
  p <- fit2[[i]]$par
  print(paste("Complete spectra nr.", i))
}

p[5]<-p[5]-70
p[8]<-p[8]-8
for (i in 120:220){
  fit2[[i]] <- EIS_fitting(filter(select(data.2$eis[[i]], f, Re, Im)), ecm, p, pl, pu)
  p <- fit2[[i]]$par
  print(paste("Complete spectra nr.", i))
}

# Summarize the fits into one table
r_fit_1 <- lapply(seq_along(fit1), function(i){
  y<-c(z.num=i, par=fit1[[i]]$par, par_err=fit1[[i]]$par_err, E4=first(data.1$eis[[i]]$E4_p), cyc.n=first(data.1$eis[[i]]$cyc.n))
  y$state <- first(data.1$eis[[i]]$state)
  if(is.null(fit1[[i]]$par) == FALSE){x <-y}else{x<-NULL}
}) %>% 
  bind_rows()
r_fit_2 <- lapply(seq_along(fit2), function(i){
  y<-c(z.num=i, par=fit2[[i]]$par, par_err=fit2[[i]]$par_err, E4=first(data.2$eis[[i]]$E4_p), cyc.n=first(data.2$eis[[i]]$cyc.n))
  y$state <- first(data.2$eis[[i]]$state)
  if(is.null(fit2[[i]]$par) == FALSE){x <-y}else{x<-NULL}
}) %>% 
  bind_rows()

## ICI analysis of both cells in the subsequent ICI cycles ##
# Skip this part and run it alone if R terminates due to memory constraints.
# Initialize cluster
cl <- makeCluster(detectCores())
clusterEvalQ(cl, library(tidyverse))

# Only some cycles are selected here to save computing time. 
# The filter can be removed to analyze all cycles but that is memory-demanding.
ici<- bind_rows(
  mutate(ICI(filter(data.1_ici$raw, cyc.n %in% c(1,10,20,30,50)), tmin = 1, tmax = 5, three.electrode = TRUE), ident = "cell 1"),
  mutate(ICI(filter(data.2_ici$raw, cyc.n %in% c(1,10,20,30,50)), tmin = 1, tmax = 5, three.electrode = TRUE), ident = "cell 2")) %>% 
  mutate(cyc.n = cyc.n +5) # 3 precycles and 2 GITT cycles

# Close the cluster
stopCluster(cl)

#### Plotting ####
# labels (Please run this part before every plot.)
labs<-c("charge", "discharge")
names(labs)<-c("C","D")
cycn<-c("cycle 1","cycle 2")
names(cycn)<-c(1, 2)

## k-E plots in Figure 3, Supplement Fig. 2 and 6 
# ('GITT 5-40s' is labeled as 'GITT' in the main text)
# Fig. 3 
plot2cell1 <- mutate(gitt_1, E = E0, k = s/I, k_err = s_err/I, method = "GITT") %>% 
  select(E, k, k_err, cyc.n, state, method)
plot2cell1 <- select(gitt_r_1, E = E4, k = k_p, k_err = k_p_err, cyc.n, state) %>% 
  mutate(method = "ICI") %>% 
  bind_rows(plot2cell1)
plot2cell1 <- mutate(r_fit_1, k_p=par8*sqrt(8/pi) , k_p_err=par_err8*sqrt(8/pi)) %>% 
  select(E = E4, k = k_p, k_err = k_p_err, cyc.n, state) %>%
  mutate(method = "EIS") %>% 
  bind_rows(plot2cell1)
plot2cell1$method <- factor(plot2cell1$method, levels = c("GITT", "ICI", "EIS"))

ggplot(filter(plot2cell1, cyc.n %in% c(1)))+
  geom_point(aes(x =E, y = k, color=method, shape = method), alpha = 1, size = 1.5) +
  geom_errorbar(aes(x =E, y = k, ymin=k-k_err,ymax=k+k_err, color=method), alpha = 0.7, size = 0.8, width=0.02) +
  labs(x = "E / V vs Li/Li"^"+"~"", y = "k / "~Omega~" s"^-0.5~"") +
  coord_cartesian(xlim = c(NA, 4.31), ylim = c(0,30))+
  scale_color_manual(values = c("#1f78b4", "#33a02c", "#ff7f00"))+
  facet_grid(.~state, labeller = labeller(state=labs))+
  theme_Lacey2()

# Supp. Fig. 2
plot2cell1 <- mutate(gitt_1, E = E0, k = s/I, k_err = s_err/I, method = "GITT 5-40 s") %>% 
  select(E, k, k_err, cyc.n, state, method)
plot2cell1 <- mutate(gitt_1_150, E = E0, k = s/I, k_err = s_err/I, method = "GITT 50-150 s") %>% 
  select(E, k, k_err, cyc.n, state, method) %>% 
  bind_rows(plot2cell1)
plot2cell1 <- select(gitt_r_1, E = E4, k = k_p, k_err = k_p_err, cyc.n, state) %>% 
  mutate(method = "ICI") %>% 
  bind_rows(plot2cell1)
plot2cell1 <- mutate(r_fit_1, k_p=par8*sqrt(8/pi) , k_p_err=par_err8*sqrt(8/pi)) %>% 
  select(E = E4, k = k_p, k_err = k_p_err, cyc.n, state) %>%
  mutate(method = "EIS") %>% 
  bind_rows(plot2cell1)
plot2cell1$method <- factor(plot2cell1$method, levels = c("GITT 5-40 s", "GITT 50-150 s", "ICI", "EIS"))

ggplot(plot2cell1)+
  geom_point(aes(x =E, y = k, color=method), alpha = 1, size = 1.5) +
  geom_errorbar(aes(x =E, y = k, ymin=k-k_err,ymax=k+k_err, color=method), alpha = 0.7, size = 0.8, width=0.02) +
  labs(x = "E / V", y = "k / "~Omega~" s"^-0.5~"") +
  scale_y_continuous(limits = c(NA,30))+
  scale_color_manual(values = c("#1f78b4", "#a6cee3", "#33a02c", "#b2df8a"))+
  facet_grid(cyc.n~state, labeller = labeller(state=labs, cyc.n=cycn))+
  theme_Lacey2()+
  NULL

# Supp. Fig. 6
plot2cell2 <- mutate(gitt_2, E = E0, k = s/I, k_err = s_err/I, method = "GITT 5-40 s") %>% 
  select(E, k, k_err, cyc.n, state, method)
plot2cell2 <- mutate(gitt_2_150, E = E0, k = s/I, k_err = s_err/I, method = "GITT 50-150 s") %>% 
  select(E, k, k_err, cyc.n, state, method) %>% 
  bind_rows(plot2cell2)
plot2cell2 <- select(gitt_r_2, E = E4, k = k_p, k_err = k_p_err, cyc.n, state) %>% 
  mutate(method = "ICI") %>% 
  bind_rows(plot2cell2)
plot2cell2 <- mutate(r_fit_2, k_p=par8*sqrt(8/pi) , k_p_err=par_err8*sqrt(8/pi)) %>% 
  select(E = E4, k = k_p, k_err = k_p_err, cyc.n, state) %>%
  mutate(method = "EIS") %>% 
  bind_rows(plot2cell2)
plot2cell2$method <- factor(plot2cell2$method, levels = c("GITT 5-40 s", "GITT 50-150 s", "ICI", "EIS"))

ggplot(plot2cell2)+
  geom_point(aes(x =E, y = k, color=method), alpha = 1, size = 1.5) +
  geom_errorbar(aes(x =E, y = k, ymin=k-k_err,ymax=k+k_err, color=method), alpha = 0.7, size = 0.8, width=0.02) +
  labs(x = "E / V", y = "k / "~Omega~" s"^-0.5~"") +
  scale_y_continuous(limits = c(NA,30))+
  scale_color_manual(values = c("#1f78b4", "#a6cee3", "#33a02c", "#b2df8a"))+
  facet_grid(cyc.n~state, labeller = labeller(state=labs, cyc.n=cycn))+
  theme_Lacey2()+
  NULL

## Slope of the OCP (dE_OC/dt_I) vs E in Figure 4 and Supplementary Fig. 4
scientific_10 <- function(x) {   parse(text=gsub("e\\+*", " %.% 10^", x)) }
plot4 <- mutate(gitt_r_1, dEdt = dE0_pdt_i, method = "ICI", ident = "cell 1") %>%
  select(E=E4, dEdt, method, state, cyc.n, ident)
plot4 <- mutate(gitt_1, dEdt = dE_q/tp, method = "GITT", ident = "cell 1") %>%
  select(E=E4, dEdt, method, state, cyc.n, ident) %>% 
  bind_rows(plot4)
plot4 <- mutate(gitt_r_2, dEdt = dE0_pdt_i, method = "ICI", ident = "cell 2") %>%
  select(E=E4, dEdt, method, state, cyc.n, ident) %>% 
  bind_rows(plot4)
plot4 <- mutate(gitt_2, dEdt = dE_q/tp, method = "GITT", ident = "cell 2") %>%
  select(E=E4, dEdt, method, state, cyc.n, ident) %>% 
  bind_rows(plot4)

# Fig. 4
filter(plot4, cyc.n == 1, ident == "cell 1") %>% 
  ggplot()+
  geom_point(aes(x =E, y = dEdt, color = method, shape = method), alpha = 0.7, size = 1.5)+
  scale_y_continuous(limits = c(-5e-5,5e-5), label = scientific_10)+
  scale_x_continuous(limits = c(3.5,4.3), breaks = seq(3.5,4.3,by=0.2))+
  scale_color_manual(values = c("#1f78b4", "#33a02c"))+
  labs(x = "E / V vs Li/Li"^"+"~"", y = expression(paste(Delta,"E"[OC]~"/",Delta,"t"[I]~" / V s"^-1~""))) +
  theme_Lacey2()

# Supp. Fig. 4
ggplot(plot4)+
  geom_point(aes(x =E, y = dEdt, color = method), alpha = 0.7, size = 1.5)+
  scale_y_continuous(limits = c(-1e-4,1e-4), label = scientific_10)+
  scale_x_continuous(limits = c(3.5,4.3), breaks = seq(3.5,4.3,by=0.2))+
  scale_color_manual(values = c("#1f78b4", "#33a02c"))+
  labs(x = "E / V", y = "dE"[OC]~"/dt"[I]~" / V s"^-1~"", color=" ") +
  facet_grid(ident~cyc.n, labeller = labeller(state=labs, cyc.n=cycn))+
  theme_Lacey2()+
  NULL

## D-E plots in Figure 5, Supplementary Fig. 1 and 5 
# ('GITT 5-40s' is labeled as 'GITT' in the main text)
# Fig. 5
plotcell1 <- select(gitt_1, rest, E = E0, Dr = Dr, Dr_err = Dr_err, cyc.n, state) %>% 
  mutate(method = "GITT 5-40 s")
plotcell1 <- select(gitt_r_1, rest, E = E4, Dr = Dr_p , Dr_err = Dr_p_err, cyc.n, state) %>% 
  mutate(method = "ICI") %>% 
  bind_rows(plotcell1)

ggplot(filter(plotcell1, cyc.n == 1))+
  geom_point(aes(x =E, y = constant*Dr, color=method, shape = method), alpha = 1, size = 1.5) +
  geom_errorbar(aes(x =E, y = constant*Dr, ymin=constant*(Dr-Dr_err),ymax=constant*(Dr+Dr_err), color=method), alpha = 0.5, size = 0.3, width=0.03) +
  labs(x = "E / V vs Li/Li"^"+"~"", y = "D / cm"^2~" s"^-1~"") +
  scale_y_log10(labels=trans_format("log10", math_format(10^.x)), breaks=c(1E-11,1E-12,1E-13,1E-14,1E-15))+
  coord_cartesian(xlim = c(NA,4.31), ylim = c(1e-13,NA))+
  scale_color_manual(values = c("#1f78b4", "#33a02c", "#b2df8a"))+
  facet_grid(.~state, labeller = labeller(state = labs))+
  theme_Lacey2()+
  NULL

# Supp. Fig. 1
plotcell1 <- select(gitt_1_150, rest, E = E0, Dr = Dr, Dr_err = Dr_err, cyc.n, state) %>% 
  mutate(method = "GITT 50-150 s") %>% 
  bind_rows(plotcell1)
ggplot(plotcell1)+
  geom_point(aes(x =E, y = constant*Dr, color=method), alpha = 1, size = 1.5) +
  geom_errorbar(aes(x =E, y = constant*Dr, ymin=constant*(Dr-Dr_err),ymax=constant*(Dr+Dr_err), color=method), alpha = 0.5, size = 0.8, width=0.02) +
  labs(x = "E / V", y = "D / cm"^2~" s"^-1~"") +
  scale_y_log10(labels=trans_format("log10", math_format(10^.x)), breaks=c(1E-11,1E-12,1E-13,1E-14,1E-15))+
  scale_color_manual(values = c("#1f78b4", "#a6cee3", "#33a02c", "#b2df8a"))+
  facet_grid(cyc.n~state, labeller = labeller(state=labs, cyc.n=cycn))+
  theme_Lacey2()+
  NULL

# Supp. Fig. 5
plotcell2 <- select(gitt_2, rest, E = E0, Dr = Dr, Dr_err = Dr_err, cyc.n, state) %>% 
  mutate(method = "GITT 5-40 s")
plotcell2 <- select(gitt_2_150, rest, E = E0, Dr = Dr, Dr_err = Dr_err, cyc.n, state) %>% 
  mutate(method = "GITT 50-150 s") %>% 
  bind_rows(plotcell2)
plotcell2 <- select(gitt_r_2, rest, E = E4, Dr = Dr_p , Dr_err = Dr_p_err, cyc.n, state) %>% 
  mutate(method ="ICI") %>% 
  bind_rows(plotcell2)
ggplot(plotcell2)+
  geom_point(aes(x =E, y = constant*Dr, color=method), alpha = 1, size = 1.5) +
  geom_errorbar(aes(x =E, y = constant*Dr, ymin=constant*(Dr-Dr_err),ymax=constant*(Dr+Dr_err), color=method), alpha = 0.5, size = 0.8, width=0.02) +
  labs(x = "E / V", y = "D / cm"^2~" s"^-1~"") +
  scale_y_log10(labels=trans_format("log10", math_format(10^.x)), breaks=c(1E-11,1E-12,1E-13,1E-14,1E-15))+
  scale_color_manual(values = c("#1f78b4", "#a6cee3", "#33a02c", "#b2df8a"))+
  facet_grid(cyc.n~state, labeller = labeller(state=labs, cyc.n=cycn))+
  theme_Lacey2()+
  NULL

## R-E plots in Figure 6, Supplementary Fig. 3 and 7 
# ('GITT 5-40s' is labeled as 'GITT' in the main text)
# Fig. 6
plot3cell1 <- mutate(gitt_1, E = E0, method = "GITT 5-40 s") %>% 
  select(E, R, R_err, cyc.n, state, method)
plot3cell1 <- select(gitt_r_1, E = E4, R = R_p, R_err = R_p_err, cyc.n, state) %>% 
  mutate(method = "ICI") %>% 
  bind_rows(plot3cell1)
plot3cell1 <-mutate(r_fit_1, R = par1+par2+par5, R_err = sqrt(par_err1^2+par_err2^2+par_err5^2)) %>% 
  select(E=E4, R , R_err, cyc.n, state) %>% 
  mutate(method = "EIS") %>% 
  bind_rows(plot3cell1)
plot3cell1$method <- factor(plot3cell1$method, levels = c("GITT 5-40 s", "ICI", "EIS"))
ggplot(filter(plot3cell1, cyc.n %in% c(1)))+
  geom_point(aes(x =E, y = R, color=method, shape = method), alpha = 1, size = 1.5) +
  geom_errorbar(aes(x =E, y = R, ymin=R-R_err,ymax=R+R_err, color=method), alpha = 0.5, size = 0.3, width=0.02) +
  labs(x = "E / V vs Li/Li"^"+"~"", y = "R / "~Omega~"") +
  scale_x_continuous(limits = c(NA,NA))+
  scale_y_continuous(limits = c(0,200))+
  scale_color_manual(values = c("#1f78b4", "#33a02c", "#ff7f00"))+
  facet_grid(.~state, labeller = labeller(state=labs))+
  theme_Lacey2()

# Supp. Fig. 3
plot3cell1 <- mutate(gitt_1_150, E = E0, method = "GITT 50-150 s") %>% 
  select(E, R, R_err, cyc.n, state, method) %>% 
  bind_rows(plot3cell1)
plot3cell1$method <- factor(plot3cell1$method, levels = c("GITT 5-40 s", "GITT 50-150 s", "ICI", "EIS"))

ggplot(plot3cell1)+
  geom_point(aes(x =E, y = R, color=method), alpha = 1, size = 1.5) +
  geom_errorbar(aes(x =E, y = R, ymin=R-R_err,ymax=R+R_err, color=method), alpha = 0.5, size = 0.7, width=0.02) +
  labs(x = "E / V", y = "R / "~Omega~"") +
  scale_x_continuous(limits = c(NA,NA))+
  scale_y_continuous(limits = c(0,200))+
  scale_color_manual(values = c("#1f78b4", "#a6cee3", "#33a02c", "#b2df8a"))+
  facet_grid(cyc.n~state, labeller = labeller(state=labs, cyc.n=cycn))+
  theme_Lacey2()+
  NULL

# Supp. Fig. 7
plot3cell2 <- mutate(gitt_2, E = E0, method = "GITT 5-40 s") %>% 
  select(E, R, R_err, cyc.n, state, method)
plot3cell2 <- mutate(gitt_2_150, E = E0, method = "GITT 50-150 s") %>% 
  select(E, R, R_err, cyc.n, state, method) %>% 
  bind_rows(plot3cell2)
plot3cell2 <- select(gitt_r_2, E = E4, R = R_p, R_err = R_p_err, cyc.n, state) %>% 
  mutate(method = "ICI") %>% 
  bind_rows(plot3cell2)
plot3cell2 <- mutate(r_fit_2, R = par1+par2+par5, R_err = sqrt(par_err1^2+par_err2^2+par_err5^2)) %>% 
  select(E=E4, R , R_err, cyc.n, state) %>% 
  mutate(method = "EIS") %>% 
  bind_rows(plot3cell2)
plot3cell2$method <- factor(plot3cell2$method, levels = c("GITT 5-40 s", "GITT 50-150 s", "ICI", "EIS"))

ggplot(plot3cell2)+
  geom_point(aes(x =E, y = R, color=method), alpha = 1, size = 1.5) +
  geom_errorbar(aes(x =E, y = R, ymin=R-R_err,ymax=R+R_err, color=method), alpha = 0.7, size = 0.8, width=0.02) +
  labs(x = "E / V", y = "R / "~Omega~"") +
  scale_x_continuous(limits = c(NA,NA))+
  scale_y_continuous(limits = c(0,200))+
  scale_color_manual(values = c("#1f78b4", "#a6cee3", "#33a02c", "#b2df8a"))+
  facet_grid(cyc.n~state, labeller = labeller(state=labs, cyc.n=cycn))+
  theme_Lacey2()+
  NULL

## Figure 7
# a: D-E plots in the subsequent ICI cycles
ggplot(filter(ici, ident =="cell 1", cyc.n %in% c(6,15,25,35,55), E_p > 3.7))+
  geom_errorbar(aes(x =Q, y = constant*Dr_p, color=cyc.n, ymin = constant*(Dr_p+Dr_p_err), ymax = constant*(Dr_p-Dr_p_err)),size=1.5, width = 5, alpha = 0.5)+
  geom_point(aes(x =Q, y = constant*Dr_p, color=cyc.n), alpha = 0.7, size = 1.5) +
  labs(x = "Q / mAh g"^-1~"", y = "D / cm"^2~" s"^-1~"", color="cycle number  ") +
  #scale_x_continuous(limits = c(3.7,NA), breaks = seq(3.7,4.3,by=0.2),expand = c(0,0.02))+
  scale_y_log10(labels=trans_format("log10", math_format(10^.x)),limits=c(7E-13,0.9E-10),breaks=c(1E-10,1E-11,1E-12))+
  scale_color_viridis_c(breaks=c(6,15,25,35,55))+
  facet_grid(.~state, labeller = labeller(state=labs))+
  theme_Lacey2()+
  NULL

# b: R-E plots in the subsequent ICI cycles
ggplot(filter(ici, ident =="cell 1", cyc.n %in% c(6,15,25,35,55), E_p > 3.7))+
  geom_point(aes(x =Q, y = R_p, color=cyc.n), alpha = 0.7, size = 1.5) +
  geom_errorbar(aes(x =Q, y = R_p, color=cyc.n, ymin = R_p-R_p_err, ymax = R_p+R_p_err),size=1.5, width = 5, alpha = 0.5)+
  labs(x = "Q / mAh g"^-1~"", y = "R / "~Omega~"", color="cycle number") +
  #scale_x_continuous(limits = c(3.7,NA), breaks = seq(3.7,4.3,by=0.2),expand = c(0,0.02))+
  #scale_y_continuous(limits = c(0,170))+
  scale_color_viridis_c(breaks=c(6,15,25,35,55), limits=c(1,55))+
  facet_grid(.~state, labeller = labeller(state=labs))+
  theme_Lacey2()

## Nyquist plots in Supplementary Fig. 10
# a
ggplot(filter(bind_rows(data.1$eis), z.num %in% c(50,55,70,93)))+ #measurement numbers (z.num) can be changed to plot other measurements
  geom_point(aes(x = Re, y= -Im, color=E4_p))+
  coord_fixed()+
  scale_color_viridis_c(name="E / V", breaks=c(3.7,3.9,4.1))+
  labs(x="Z' / "~Omega~"", y="Z'' / "~Omega~"")+
  theme_Lacey2()+
  NULL

# b
ggplot(filter(bind_rows(data.1$eis), z.num %in% c(94,105,108,110)))+ #measurement numbers (z.num) can be changed to plot other measurements
  geom_point(aes(x = Re, y= -Im, color=E4_p))+
  coord_fixed()+
  scale_color_viridis_c(name="E / V", breaks=c(3.7,3.9,4.1))+
  labs(x="Z' / "~Omega~"", y="Z'' / "~Omega~"")+
  theme_Lacey2()+
  NULL

## Relative difference between results from GITT and ICI methods in Supplementary Fig. 9, 11 and 12
diff_1 <- filter(gitt_1, rest !=124, rest != 65) %>% 
  select(rest, state, cyc.n, I, E0, E4, s, s_err, Dr, Dr_err, dE_q, tp) %>% 
  mutate(k_g = s/I, k_g_err = s_err/abs(I), Dr_g = Dr, Dr_g_err = Dr_err, dEdt = dE_q / tp)

diff_1 <- merge(diff_1, select(filter(gitt_r_1), rest, k_p, k_p_err, Dr_p, Dr_p_err, dE0_pdt_i), by = "rest")
diff_1$dk <- diff_1$k_p - diff_1$k_g
diff_1$rel_dk <- diff_1$dk / diff_1$k_g
diff_1$dDr <- diff_1$Dr_p - diff_1$Dr_g
diff_1$rel_dDr <- diff_1$dDr / diff_1$Dr_g
diff_1$ddEdt <- diff_1$dE0_pdt_i - diff_1$dEdt
diff_1$rel_ddEdt <- diff_1$ddEdt / diff_1$dEdt

# Supp. Fig. 9
filter(diff_1, E4>3.7, cyc.n ==1) %>% 
  ggplot()+
  geom_point(aes(x =E4, y = rel_dk))+
  geom_hline(aes(yintercept=mean(rel_dk)), color = "#d95f02", size = 1)+
  geom_rect(aes(ymin=mean(rel_dk)-sd(rel_dk),ymax=mean(rel_dk)+sd(rel_dk),xmin=3.7,xmax=4.3), fill = "#d95f02", alpha = 0.005)+
  labs(x="E / V", y="(k"[ICI]~"-k"[GITT]~")/k"[GITT]~"")+
  scale_y_continuous(breaks = seq(-1,1,by=0.2))+
  coord_cartesian(ylim = c(-1, 1), xlim = c(3.7, 4.3))+
  theme_Lacey2()

# Supp. Fig. 11
filter(diff_1, E4>3.65) %>% 
  ggplot()+
  geom_point(aes(x =E4, y = rel_ddEdt))+
  geom_hline(aes(yintercept=mean(rel_ddEdt)), color = "#d95f02", size = 1)+
  geom_rect(aes(ymin=mean(rel_ddEdt)-sd(rel_ddEdt),ymax=mean(rel_ddEdt)+sd(rel_ddEdt),xmin=3.65,xmax=4.3), fill = "#d95f02", alpha = 0.005)+
  coord_cartesian(xlim = c(3.65, NA), ylim = c(-1,1))+
  scale_y_continuous(breaks = seq(-1,1,by=0.2))+
  labs(x="E / V", y="(slope"[ICI]~"-slope"[GITT]~")/slope"[GITT]~"")+
  theme_Lacey2()

# Supp. Fig. 12
filter(diff_1, E4>3.7, cyc.n ==1) %>% 
  ggplot(aes(x =E4, y = rel_dDr))+
  geom_point()+
  geom_hline(aes(yintercept=mean(rel_dDr)), color = "#d95f02", size = 1)+
  geom_rect(aes(ymin=mean(rel_dDr)-sd(rel_dDr),ymax=mean(rel_dDr)+sd(rel_dDr),xmin=3.7,xmax=4.3), fill = "#d95f02", alpha = 0.005)+
  coord_cartesian(ylim = c(-0.8, 1.4))+
  scale_y_continuous(breaks = seq(-0.8,1.4,by=0.2))+
  labs(x="E / V", y="(D"[ICI]~"-D"[GITT]~")/D"[GITT]~"")+
  theme_Lacey2()