require(tidyverse)

#### signCurrent function ###
signCurrent <- function(x){
  ifelse(x > 0, "C", ifelse(x < 0, "D", "R"))
  }
  

#### assignRest function ###
assignRest <- function(x){
  
  require(dplyr)
  
  y <- lag(x)
  y[1] <- x[1]
  a <- ifelse(x != "R" & y == "R",1,0)
  b <- cumsum(a) + 1 #just to let it start from 1
  return(b)
}

#### assignCycle function ###
assignCycle <- function(x, skip.first = FALSE){
  require(dplyr)
  
  y <- tibble(state = x, count = rep(0, length(x))) %>%
        rowid_to_column("sequence")
  z <- filter(y, state != "R")
  z$count <- ifelse(z$state != lag(z$state),1,0)
  z$count[1] <- 0
  z <- bind_rows(z, filter(y, state == "R"))
  z <- arrange(z, sequence)
  z$count <- cumsum(z$count)
  if(skip.first == TRUE){z$count <- z$count - 1}
  cyc <- abs(z$count) %/% 2 +1
  return(cyc)
}

### Import data generated by Bio-Logic ###
import_biologic <- function(file, data = NULL, step.time = TRUE, mass = NULL, mass.err = 0.02, electr.err = 0.025, skip.1st.half.cycle = FALSE, three.electrode = FALSE) { 
  #We estimate the error in weighing the coating to be 0.02 mg and the error associated with the thickness variation of the current collector to be 0.025 mg
  
  require(dplyr)
  if(is.null(data)){
  if(str_sub(file, start = -3) == "mpt"){
      # read in the data from mpt files
      sd <- file %>% 
      readLines(n = 2) %>% .[2] %>% 
      gsub("Nb header lines : ", "", .) %>%
      as.numeric %>% 
      -1 %>% 
      read_tsv(file, skip = . )
  }else{
    stop("File type not supported.")
  }}else{
    sd <- data
  }
      
  if("freq/Hz" %in% colnames(sd)){
        sde <- filter(sd, sd$'freq/Hz' != 0) %>% 
                select(t = 'time/s',
                       f = 'freq/Hz',
                       Re_cell ='Re(Zwe-ce)/Ohm',
                       Im_cell = '-Im(Zwe-ce)/Ohm',
                       Re_n ='Re(Zce)/Ohm',
                       Im_n = '-Im(Zce)/Ohm',
                       z.cyc = 'z cycle',
                       Ns = Ns
                       ) %>% 
                mutate(Im_cell = - Im_cell, Im_n = - Im_n, Re = Re_cell - Re_n, Im = Im_cell - Im_n)
        sd <- filter(sd, sd$'freq/Hz' == 0)
      }
      
  # list of variables to select - 2-electrode case
    if(three.electrode == FALSE) {
        sd <- select(sd, t = 'time/s',
                     step.n = Ns,
                     I = 'I/mA',
                     E = 'Ewe/V',
                     Q = '(Q-Qo)/mA.h'
        )   
      }
      
      # list of variables to select - 3-electrode case
      if(three.electrode == TRUE) {
        sd <- select(sd, t = 'time/s',
                     step.n = Ns,
                     I = 'I/mA',
                     E_p = 'Ewe/V',
                     E_n = 'Ece/V',
                     Q = '(Q-Qo)/mA.h'
        ) %>% 
          mutate(E = E_p - E_n)
      }

  sd <- sd %>%
    mutate(state = signCurrent(I), rest = assignRest(state), cyc.n = assignCycle(state, skip.first = skip.1st.half.cycle), I = I*0.001)
  
  # reset Q
  sd <- sd %>%
    group_by(cyc.n) %>%
    mutate(Q = Q - Q[1]) %>%
    ungroup
  
  # Step time is included if specified.
  if(step.time == TRUE) {
    sd <- sd %>%
      group_by(rest, state) %>%
      mutate(step.t = t - t[1]) %>%
      ungroup
  }

  stats <- sd %>%
    group_by(cyc.n, state) %>%
    summarise(Q = max(Q)-min(Q)) %>%
    spread(key = state, value = Q) %>%
    ungroup()

  # Deal with only one incomplete cycle
  if("C" %in% colnames(stats) & "D" %in% colnames(stats)){
    stats<-select(stats, cyc.n, Q.c = C, Q.d = D)
  }else if("C" %in% colnames(stats)){
    stats<-mutate(select(stats, cyc.n, Q.c = C), Q.d = NA)
  }else{
    stats<-mutate(select(stats, cyc.n, Q.d = D), Q.c = NA)
  }

  stats$CE <- stats$Q.d / stats$Q.c
  
  if(!is.null(mass) == TRUE) {
    sd$Q <- sd$Q * 1000 /mass
    stats$Q.c <- stats$Q.c * (1000/mass)
    stats$Q.c.err <- stats$Q.c * (mass.err + electr.err) / mass
    stats$Q.d <- stats$Q.d * (1000/mass)
    stats$Q.d.err <- stats$Q.d * (mass.err + electr.err) / mass
  }
  
  if(first(filter(sd, cyc.n == ifelse(max(sd$cyc.n)>1, 2, 1), state != "R")$state) == "D") {
    sd$Q <- sd$Q * -1
  }
  
  if(exists("sde") == FALSE){
    out <- list(raw = sd, stats = stats)
  }else{
    sde <- group_by(sde, z.cyc, Ns) %>% 
            mutate(g.id = cur_group_id()) %>% 
            ungroup() %>% 
            arrange(t) %>% 
            mutate(seq.g.id = if_else(g.id == lag(g.id, default = 1), 0, 1), z.num =cumsum(seq.g.id)) %>% 
            select(-g.id, -seq.g.id, -Ns, -z.cyc)
          
    sdf <- group_by(sde, z.num) %>% 
      summarise(t1 = first(t))
    sdg <- lapply(sdf$t1, function(x){
      out <- filter(sd, t == max(filter(sd, t<x)$t))
        }) %>% 
      bind_rows() %>% 
      mutate(z.num = sdf$z.num) %>% 
      select(E4 = E, E4_p = E_p, E4_n = E_n, z.num, t0 = t, Q, rest, cyc.n)
    sdg$state <- sapply(sdg$rest, function(x) last(filter(sd, state != "R", rest == x)$state))
    sde <- merge(sde, sdg, by = "z.num")
    sde <- group_by(sde, z.num) %>% 
            arrange(desc(f)) %>% 
            group_split()
    out <- list(raw = sd, stats = stats, eis = sde)
  }
  return(out)}


# ICI

ICI <- function(data.in, ncores = detectCores(), three.electrode = FALSE, tmin = 0.1, tmax = 0.9) {
  
  require(dplyr)
  require(parallel)
  
  raw <- data.in
  message(paste("Now assembling processed data, please wait a moment... "), appendLF = FALSE)
  
  if(length(unique(filter(raw, cyc.n == max(cyc.n))$rest))<2){
    raw <- filter(raw, cyc.n < max(cyc.n))
  }                                                                #Take away the last cycle if it does not contain current pause
  
  clusterExport(cl, "raw")
  
  proc <- parLapply(cl, split(raw, raw$cyc.n), function(cycle) {
    
    if(last(cycle$state) != "R") {
      cycle <- filter(cycle, rest < max(cycle$rest))
    }
    
    if(first(cycle$state) == "R") {
      cycle <- filter(cycle, rest > first(cycle$rest))
    }
  
    summary <- cycle %>%
      group_by(rest) %>%
      filter(state != "R") %>%
      summarise_all(last) %>% 
      mutate(t_i = cumsum(step.t))
    
    # Perform regression on each rest, time-filtered as required, and return results
    regression <- cycle %>%
      group_by(rest) %>%
      filter(state == "R", step.t >= tmin, step.t <= tmax) %>%
      group_modify(~ lm(E ~ sqrt(step.t), data = .x) %>%
                     with(
                       data.frame(
                         E0 = coef(summary(.))[1, 1],                
                         E0_err = coef(summary(.))[1, 2], #and error
                         s = coef(summary(.))[2, 1], #and gradient
                         s_err = coef(summary(.))[2, 2] #and error
                       )))
    
    if(three.electrode==FALSE){
      
      # Merge the two halves of the data by rest, calculate R and k and remove unwanted columns
      out <- merge(summary, regression, by = "rest") %>%
        mutate(R = (E - E0) / I, R_err = E0_err / abs(I), k = -s / I, k_err = s_err / abs(I), dE0dt_i = (E0 - lag(E0))/ (t_i-lag(t_i)), Dr = (dE0dt_i/s)^2) %>%
        select(-step.t, -s, -s_err, -step.n)
      
    }else{
      
      # WE-RE
      regression_p <- cycle %>%
        group_by(rest) %>%
        filter(state == "R", step.t >= tmin, step.t <= tmax) %>%
        group_modify(~ lm(E_p ~ sqrt(step.t), data = .x) %>%
                       with(
                         data.frame(
                           E0_p = coef(summary(.))[1, 1],                
                           E0_p_err = coef(summary(.))[1, 2], #and error
                           s_p = coef(summary(.))[2, 1], #and gradient
                           s_p_err = coef(summary(.))[2, 2] #and error
                         )))
      
      # CE-RE
      regression_n <- cycle %>%
        group_by(rest) %>%
        filter(state == "R", step.t >= tmin, step.t <= tmax) %>%
        group_modify(~ lm(E_n ~ sqrt(step.t), data = .x) %>%
                       with(
                         data.frame(
                           E0_n = coef(summary(.))[1, 1],                
                           E0_n_err = coef(summary(.))[1, 2], #and error
                           s_n = coef(summary(.))[2, 1], #and gradient
                           s_n_err = coef(summary(.))[2, 2] #and error
                         )))
      
      # Merge the two halves of the data by rest, calculate R and k and remove unwanted columns
      out <- merge(summary, regression, by = "rest") %>%
        merge(. ,regression_p, by = "rest") %>%
        merge(. ,regression_n, by = "rest") %>%
        mutate(R = (E - E0) / I, R_err = E0_err / abs(I), k = -s / I , k_err = s_err / abs(I), dE0dt_i = (E0 - lag(E0))/(t_i-lag(t_i)), Dr = (dE0dt_i/s)^2, Dr_err = Dr*2*sqrt((E0_err/E0)^2+(s_err/s)^2), 
               R_p = (E_p - E0_p) / I , R_p_err = E0_p_err / abs(I), k_p = -s_p / I , k_p_err = s_p_err / abs(I), dE0_pdt_i = (E0_p - lag(E0_p))/ (t_i-lag(t_i)), Dr_p = (dE0_pdt_i/s_p)^2, Dr_p_err = Dr_p*2*sqrt((E0_p_err/E0_p)^2+(s_p_err/s_p)^2),
               R_n = -1 * (E_n - E0_n) / I , R_n_err = E0_n_err / abs(I), k_n = s_n / I , k_n_err = s_n_err / abs(I), dE0_ndt_i = (E0_n - lag(E0_n))/ (t_i-lag(t_i)), Dr_n = (dE0_ndt_i/s_n)^2, Dr_n_err = Dr_n*2*sqrt((E0_n_err/E0_n)^2+(s_n_err/s_n)^2))  %>%
        select(-step.t, -s, -s_err, -s_p, -s_p_err, -s_n, -s_n_err, -step.n)
      
      
    }
    return(out)
  })
  
  proc <- bind_rows(proc)
  message("done.")  
  return(proc)
}

# ICI with diffusion coefficient in non-parallel

ICI_np <- function(data.in, three.electrode = FALSE, tmin = 0.1, tmax = 0.9) {
  
  require(dplyr)
  
  raw <- data.in
  message(paste("Now assembling processed data, please wait a moment... "), appendLF = FALSE)
  
  if(length(unique(filter(raw, cyc.n == max(cyc.n))$rest))<2){
    raw <- filter(raw, cyc.n < max(cyc.n))
  }                                                                #Take away the last cycle if it does not contain current pause
  

  summary <- raw %>%
      group_by(rest) %>%
      filter(state != "R") %>%
      summarise_all(last) %>% 
      mutate(t_i = cumsum(step.t))
    
  # Perform regression on each rest, time-filtered as required, and return results
  regression <- raw %>%
      group_by(rest) %>%
      filter(state == "R", step.t >= tmin, step.t <= tmax) %>%
      group_modify(~ lm(E ~ sqrt(step.t), data = .x) %>%
                     with(
                       data.frame(
                         E0 = coef(summary(.))[1, 1],                
                         E0_err = coef(summary(.))[1, 2], #and error
                         s = coef(summary(.))[2, 1], #and gradient
                         s_err = coef(summary(.))[2, 2] #and error
                       )))
    
  if(three.electrode==FALSE){
      
      # Merge the two halves of the data by rest, calculate R and k and remove unwanted columns
      out <- merge(summary, regression, by = "rest") %>%
        mutate(R = (E - E0) / I, R_err = E0_err / abs(I), k = -s / I, k_err = s_err / abs(I), dE0dt_i = (E0 - lag(E0))/ (t_i-lag(t_i)), Dr = (dE0dt_i/s)^2, Dr_err = Dr*2*sqrt((E0_err/E0)^2+(s_err/s)^2)) %>%
        select(-step.t, -s, -s_err, -step.n)

  }else{
      
      # WE-RE
      regression_p <- raw %>%
        group_by(rest) %>%
        filter(state == "R", step.t >= tmin, step.t <= tmax) %>%
        group_modify(~ lm(E_p ~ sqrt(step.t), data = .x) %>%
                       with(
                         data.frame(
                           E0_p = coef(summary(.))[1, 1],                
                           E0_p_err = coef(summary(.))[1, 2], #and error
                           s_p = coef(summary(.))[2, 1], #and gradient
                           s_p_err = coef(summary(.))[2, 2] #and error
                         )))
      
      # CE-RE
      regression_n <- raw %>%
        group_by(rest) %>%
        filter(state == "R", step.t >= tmin, step.t <= tmax) %>%
        group_modify(~ lm(E_n ~ sqrt(step.t), data = .x) %>%
                       with(
                         data.frame(
                           E0_n = coef(summary(.))[1, 1],                
                           E0_n_err = coef(summary(.))[1, 2], #and error
                           s_n = coef(summary(.))[2, 1], #and gradient
                           s_n_err = coef(summary(.))[2, 2] #and error
                         )))
      
      # Merge the two halves of the data by rest, calculate R and k and remove unwanted columns
      out <- merge(summary, regression, by = "rest") %>%
        merge(. ,regression_p, by = "rest") %>%
        merge(. ,regression_n, by = "rest") %>%
        mutate(R = (E - E0) / I, R_err = E0_err / abs(I), k = -s / I , k_err = s_err / abs(I), dE0dt_i = (E0 - lag(E0))/(t_i-lag(t_i)), Dr = (dE0dt_i/s)^2, Dr_err = Dr*2*sqrt((E0_err/E0)^2+(s_err/s)^2), 
               R_p = (E_p - E0_p) / I , R_p_err = E0_p_err / abs(I), k_p = -s_p / I , k_p_err = s_p_err / abs(I), dE0_pdt_i = (E0_p - lag(E0_p))/ (t_i-lag(t_i)), Dr_p = (dE0_pdt_i/s_p)^2, Dr_p_err = Dr_p*2*sqrt((E0_p_err/E0_p)^2+(s_p_err/s_p)^2),
               R_n = -1 * (E_n - E0_n) / I , R_n_err = E0_n_err / abs(I), k_n = s_n / I , k_n_err = s_n_err / abs(I), dE0_ndt_i = (E0_n - lag(E0_n))/ (t_i-lag(t_i)), Dr_n = (dE0_ndt_i/s_n)^2, Dr_n_err = Dr_n*2*sqrt((E0_n_err/E0_n)^2+(s_n_err/s_n)^2))  %>%
        select(-step.t, -s, -s_err, -s_p, -s_p_err, -s_n, -s_n_err, -step.n)
      
  }

  bind_rows(out)
  message("done.")  
  return(out)
}

# GITT

GITT <- function(data.in, tmin = 0.1, tmax = 0.9, three.electrode = FALSE) {
  
  require(dplyr)
  
  raw <- data.in
  message(paste("Now assembling processed data, please wait a moment... "), appendLF = FALSE)
  
  if(length(unique(filter(raw, cyc.n == max(cyc.n))$rest))<2){
    raw <- filter(raw, cyc.n < max(cyc.n))
  }                                                                #Take away the last cycle if it does not contain current pause
  
  if(three.electrode){raw <- mutate(raw, E = E_p)}
  
  summary <- raw %>%
    group_by(rest) %>%
    filter(state == "R") %>%
    summarise_all(last) %>%
    mutate(E4 = E, E0 = lag(E))
  
  summary <- select(summary, rest, t, E0, E4, Q, cyc.n)
  summary <- summary[-1,]
  
  if(last(raw$state) != "R"){raw <- filter(raw, rest < max(raw$rest))}
  summary2  <- raw %>%
    filter(rest>min(raw$rest)) %>%
    group_by(rest) %>%
    filter(state != "R") %>%
    summarise_all(last) %>%
    select(I,state,tp=step.t,E2=E)
  
  summary <- bind_cols(summary, summary2)

  
  # Perform regression on each rest, time-filtered as required, and return results
  regression <- raw %>%
    filter(rest>min(raw$rest), rest<max(raw$rest)) %>%
    group_by(rest) %>%
    filter(state != "R", step.t >= tmin, step.t <= tmax) %>%
    group_modify(~ lm(E ~ sqrt(step.t), data = .x) %>%
                   with(
                     data.frame(
                       E1 = coef(summary(.))[1, 1],                
                       E1_err = coef(summary(.))[1, 2], #and error
                       s = coef(summary(.))[2, 1], #and gradient
                       s_err = coef(summary(.))[2, 2] #and error
                     )))
  

    # Merge the two halves of the data by rest, calculate R and k and remove unwanted columns
    out <- merge(summary, regression, by = "rest") %>%
      mutate(dE_q = E4 - E0, Dr = (dE_q/s/tp)^2, Dr_err = Dr*2*(s_err/s), R = (E1 - E0)/I, R_err = E1_err/abs(I))
    
  message("done.")  
  return(out)
}


#### EIS_fitting function ####
EIS_fitting <- function(data.in, ecm, initial.parameters, lower.bounds, upper.bounds){
  require(minpack.lm)
  require(impedanceR)
  
  data <- data.in
  p0 <- initial.parameters
  pl <- lower.bounds
  pu <- upper.bounds
  
  # Separate the data into f, zr, and zi
  f <- data[,1,1]
  zr <- data[,2,1]
  zi <- data[,3,1]
  # Complex impedance data list
  zc <- zr + 1i*zi
  
  # Declare the residual function
  rsd <- function(p,z,x) {
    errz = ecm(p,x)-z
    err = c(Re(errz), Im(errz))
    return (err)
  }
  
  # start fitting
  nls.out <- nls.lm(p = p0, lower = pl, upper = pu, fn = rsd, z = zc, x = f, 
                    control =  nls.lm.control(nprint = 1, maxfev = 1000, maxiter = 1000, ftol = 0.000001))
  pfit <- coef(nls.out)
  
  while (summary(nls.out)[[5]] != 1){
    p0 <- pfit
    nls.out <- nls.lm(p = p0, lower = pl, upper = pu, fn = rsd, z = zc, x = f, 
                      control =  nls.lm.control(nprint = 1, maxfev = 1000, maxiter = 100, ftol = 0.000001))
  }
  summary(nls.out)
  pfit <- summary(nls.out)[[8]][,1]
  perr <- summary(nls.out)[[8]][,2]
  RSS <- sum(abs(ecm(pfit,f)-zc)*abs(ecm(pfit,f)-zc))
  #fcov <- vcov(nls.out)
  out <- list(par = pfit, par_err = perr, RSS = RSS)
  
  return(out)
}
