#### Install packages (if necessary) ####
install.packages("tidyverse")
install.packages("scales")
install.packages("viridis")
install.packages("ggthemes")

#### Packages checks ####
# Start here if packages are installed
library(tidyverse)
library(scales)
library(viridis)
library(ggthemes)

#### Get the plotting theme and functions ####
# Please make sure that the working directory is the same as the directory of this script.
source("../R_functions_themes/theme.R")
source("../R_functions_themes/functions.R")

#### IMPORT DATA ####
# Import and analyze electrochemical data during operando XRD
data.in <- import_biologic("HL01_2_C01.mpt", mass = 2.8, three.electrode = TRUE)
proc<-ICI_np(filter(data.in$raw), three.electrode = TRUE, tmin = 1, tmax = 5)
pot<-filter(data.in$raw, step.t >30)

# Constants in Equation 2: 4/pi*(V/A)^2 
V <- 0.63056 #cm3/g
A <- 1.5e4 #cm2/g
constant <- 4/pi*(V/A)^2  #cm2
 
# Import XRD patterns
pattern<-lapply(1:42, function(i){
  df<-read.table(paste("./xy/HL01_op2_back.", formatC(i,width = 3,flag = "0"), ".xy", sep=""),colClasses = "numeric")
  df$time<-(as.numeric(i)-1)*900+450
  df$nr <- paste0("HL01_op2_back.", formatC(i,width = 3,flag = "0"))
  return(df)
  })
pattern<-bind_rows(pattern)
pattern<-rename(pattern, twotheta=V1, intensity=V2)

# little function to find the voltage of each pattern
volt.patt <- group_by(pattern, time) %>% 
              summarise(first(nr)) %>% 
              ungroup
volt.patt$E_p <- sapply(volt.patt$time, function(x) pot$E_p[which.min(abs(pot$t-x))])
print(volt.patt)

#### PLOTTING ####
# The following are the panels in Figure 8
# Heat map
filter(pattern, twotheta<20, twotheta>18) %>%
  ggplot(aes(x=time/3600, y=twotheta, fill=intensity))+
  geom_raster()+
  labs(x="t / h",y=expression(paste("2",theta," / ",degree)))+
  scale_fill_viridis("Counts",limits=c(0,4000))+
  scale_x_continuous(limits = c(0, 10.25))+
  theme_Lacey2()

# E-t plot
ggplot(pot, aes(x=t/3600, y=E))+
  geom_path(size=1,linejoin = "round", lineend = "round")+
  scale_x_continuous(limits = c(0, 10.25))+
  labs(x="t / h", y="E / V")+
  theme_Lacey2()

# D-t plot
ggplot(proc, aes(x=t/3600, y= constant*Dr_p))+
  geom_path(size=1,linejoin = "round", lineend = "round")+
  geom_errorbar(aes(ymin = constant*(Dr_p-Dr_p_err), ymax = constant*(Dr_p+Dr_p_err))) +
  scale_y_log10(labels=trans_format("log10", math_format(10^.x)), breaks=c(1E-11,1E-12))+
  scale_x_continuous(limits = c(0, 10.25))+
  labs(x="t / h", y = "D / cm"^2~" s"^-1~"")+
  theme_Lacey2()

# R-t plot
ggplot(proc, aes(x = t/3600, y = R)) +
  geom_path(size=1, linejoin = "round", lineend = "round")+
  geom_errorbar(aes(ymin = (R - R_err) , ymax = (R + R_err))) +
  scale_x_continuous(limits = c(0, 10.25))+
  theme_Lacey2()+
  labs(x = "t / h", y = "R /"~Omega~"")

# XRD patterns
ggplot(filter(pattern,nr %in% c("HL01_op2_back.014")))+ #Please specify the xy file number here
  geom_path(aes(x=twotheta, y=intensity, color=time), alpha = 0.75)+
  theme_Lacey2()

